/*
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 * SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 * XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 * AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 * OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 * IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 * AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 * FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 * WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 * IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 * REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 * INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 *     
 * (c) Copyright 2006 Xilinx, Inc.
 * All rights reserved.
 *
 */
 
/*
 * Modification history:
 *    01/2006 - Rob Armstrong - Xilinx, Inc.
 *            - Initial File Version
 */

#ifndef _ETHERNET_H
#define _ETHERNET_H

#include "string.h" // Necessary for memcpy, memset, etc.

#include "ping_responder.h"

/* Constant definitions for Ethernet packet processing */
#define DEST_MAC_OFFSET 0
#define SOURCE_MAC_OFFSET 6
#define LENGTHTYPE_OFFSET 12
#define ARP_TARGET_IP_OFFSET 38
#define ARP_OPCODE_OFFSET 20
#define ARP_SOURCE_MAC_OFFSET 22
#define ARP_DEST_MAC_OFFSET 32 
#define ARP_OPCODE_REPLY 2
#define IP_PROTOCOL_OFFSET 23
#define ICMP 0x01
#define IP_ICMP_TYPE_OFFSET 34
#define IP_ICMP_CHECKSUM_OFFSET 36
#define ECHO_REQUEST 0x08
#define ECHO_REPLY 0x00
#define IP_ID_OFFSET 18
#define IP_SOURCE_IP_OFFSET 26
#define IP_HEADER_OFFSET 14
#define IP_DEST_IP_OFFSET 30
#define IP_HEADER_LENGTH 20
#define IP_HEADER_CHECKSUM_OFFSET 24
#define CRC_LENGTH 4
#define IP_DATA_OFFSET 34

/* Constant definitions for subroutine return codes */
#define NO_RETRANSMIT 0
#define RETRANSMIT 1
#define ARP_DIFFERENT_TARGET 5
#define ARP_NOT_REQUEST 4
#define ARP_RESPONSE_OK 1

/* Function Prototypes */
int processPacket(unsigned char *rxFrame,
                   unsigned char *macAddress,
                   unsigned char *ipAddress,
                   unsigned long receiveLength);
int isBroadcast(unsigned char *rxFrame);
int isUnicastMatch(unsigned char *rxFrame, unsigned char *macAddress);
int isArp(unsigned char *rxFrame);
int processArp(unsigned char *rxFrame,
               unsigned char *macAddress,
               unsigned char *ipAddress);
void swapSourceDestMac(unsigned char *rxFrame);
int isPing(unsigned char *rxFrame);
void processPing(unsigned char *rxFrame,
                 unsigned char *macAddress,
                 unsigned char *ipAddress,
                 unsigned long receiveLength);
unsigned short calcChecksum(unsigned char *data,
                            int length);


#endif
