/*
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 * SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 * XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 * AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 * OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 * IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 * AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 * FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 * WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 * IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 * REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 * INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 *     
 * (c) Copyright 2006 Xilinx, Inc.
 * All rights reserved.
 *
 */
 
/*
 * Modification history:
 *    01/2006 - Rob Armstrong - Xilinx, Inc.
 *            - Initial File Version
 */

/* Includes for Xilinx IP */


/* Includes for local application files */
#include "serial_io.h"
#include "ping_responder.h"
#include "ethernet.h"

extern void  mdio_set_rgmii();

#define PROD_TEST
#ifdef PROD_TEST
    unsigned char macAddress[6]= { 0x00, 0x0A, 0x35, 0x01, 0x02, 0x03 };
	unsigned char ipAddress[4] = { 192, 168, 0, 2 };
#endif

int main(void)
{
    XTemac mac;
    XStatus status;
#ifndef PROD_TEST
    unsigned char macAddress[6];
    unsigned char ipAddress[4];
#endif
    unsigned char rxFrame[XTE_MAX_FRAME_SIZE];
    Xuint32 receiveLength;
    Xuint32 transmitLength;
    unsigned int payloadLength;
    int loop;
    int result;
    int fifoFreeBytes;
     
        // Initialize the UART16550 to run at 9600-8-N-1
    XUartNs550_SetBaud(UART_BASEADDR, UART_CLK_SPEED, 9600);
    XUartNs550_mSetLineControlReg(UART_BASEADDR, XUN_LCR_8_DATA_BITS);
    
    clearScreen();
    printLegalHeader();
    
    // Initialize the TEMAC instance
    status = XTemac_Initialize(&mac, TEMAC_DEVICE_ID);
    if(status != XST_SUCCESS) {
        xil_printf("ERROR: Unable to initialize TEMAC instance: %d\r\n", status);
        return 0;
    }
    
    // Run a self=test on the TEMAC
    status = XTemac_SelfTest(&mac);
    if(status != XST_SUCCESS) {
        xil_printf("ERROR: TEMAC failed self test: %d\r\n", status);
        return 0;
    }

    /* MDIO  */
/******* Skipping for SGMII mode ***************************************
    xil_printf("Setting up PHY via MDIO...\r\n");
    mdio_set_rgmii();
    xil_printf("Setting up PHY via MDIO...DONE\r\n");
************************************************************************/
    
    // Read in and set the MAC address
#ifndef PROD_TEST
    getMacAddr(macAddress);
#endif
    status = XTemac_SetMacAddress(&mac, macAddress);
    if(status != XST_SUCCESS) {
        xil_printf("ERROR: Unable to set TEMAC MAC address: %d\r\n", status);
        return 0;
    }
    
    // Set the TEMAC to polled mode
    status = XTemac_SetOptions(&mac, XTE_POLLED_OPTION);
    if(status != XST_SUCCESS) {
        xil_printf("ERROR: Unable to set TEMAC options: %d\r\n", status);
        return 0;
    }
    
    // Read in the IP address to be used for responding to ARP and ICMP
    // packets, then display the configuration header  
#ifndef PROD_TEST
    while(readIP(ipAddress) != TRUE);
#endif    
    
    XTemac_mWriteHostReg(TEMAC_BASEADDR, XTE_ECFG_OFFSET,
                         (XTE_ECFG_HOSTEN_MASK | XTE_ECFG_LINKSPD_1000));

    /* Added SGMII Configuration Register Access */
    XTemac_mWriteHostReg(TEMAC_BASEADDR, XTE_EGMIC_OFFSET,
                         (XTE_EGMIC_SGLINKSPD_1000));

    xil_printf("\r\nConfiguring PHY");
    for(loop = 0; loop < 5; loop++) {
        usleep(1000000);
        xil_printf(".");
    }
    xil_printf("done\r\n");
    
    
    status = XTemac_Start(&mac);
    if(status != XST_SUCCESS) {
        xil_printf("ERROR: Unable to start TEMAC: %d\r\n", status);
        return 0;
    }
    
    clearScreen();
    displayConfiguration(macAddress, ipAddress);
    
    memset(rxFrame, 0, XTE_MAX_FRAME_SIZE);
    
    while(1) {
        // Loop until we receive a packet, then process it
        if(XTemac_FifoRecv(&mac, &receiveLength) == XST_SUCCESS) {
            xil_printf("Received frame - %d bytes\r\n", receiveLength);
            status = XTemac_FifoRead(&mac, 
                                     rxFrame, 
                                     receiveLength,
                                     XTE_END_OF_PACKET);
            if(status != XST_SUCCESS) {
                xil_printf("WARNING: Packet received but unable to read\r\n");
            }
            
            // TODO: Insert something useful here
            result = processPacket(rxFrame, macAddress, ipAddress, receiveLength);
            if(result == RETRANSMIT) {
                // Strip the CRC off of the received packet     
                transmitLength = receiveLength - CRC_LENGTH;   
                 
                // Wait for enough room in FIFO to become available
                do {
                    fifoFreeBytes = XTemac_FifoGetFreeBytes(&mac, XTE_SEND);
                } while (fifoFreeBytes < transmitLength);
    
                // Write frame data to the transmit FIFO
                status = XTemac_FifoWrite(&mac, rxFrame, transmitLength, XTE_END_OF_PACKET);
                if (status != XST_SUCCESS) {
                    xil_printf("ERROR: Unable to write to transmit FIFO - %d\r\n", status);
                    return 0;
                } 
    
                /* Initiate transmit */
                status = XTemac_FifoSend(&mac, transmitLength);
                if (status != XST_SUCCESS) {
                    xil_printf("ERROR: Transmission not successful - %d\r\n", status);
                    return;
                }
            }    
        }
    }    
}
