/*
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
 * SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
 * XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
 * AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
 * OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
 * IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
 * AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
 * FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
 * WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
 * IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
 * REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
 * INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 *     
 * (c) Copyright 2006 Xilinx, Inc.
 * All rights reserved.
 *
 */
 
/*
 * Modification history:
 *    01/2006 - Rob Armstrong - Xilinx, Inc.
 *            - Initial File Version
 */

/* Xilinx Includes */
#include "ping_responder.h"
#include "stdlib.h"
 
/* void printLegalHeader(void)
 * 
 * Displays the Xilinx reference design legal header to stdout.
 */
void printLegalHeader(void)
{
	xil_printf("####################################################################\r\n");
	xil_printf("# Xilinx ICMP Echo (Ping) Demo Application v1.0                    #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("# XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"    #\r\n");
	xil_printf("# SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR          #\r\n");
	xil_printf("# XILINX DEVICES.                                                  #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("# (c) Copyright 2006 Xilinx, Inc.                                  #\r\n");
	xil_printf("# All rights reserved                                              #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("####################################################################\r\n");
	xil_printf("\n\r\n");
}

/* void printByteDashedHex(unsigned char* buffer, int length)
 * 
 * Displays the contents of a byte array as a dash-delimited string
 * to STDOUT
 */
void printByteDashedHex(unsigned char* buffer, int length)
{
    unsigned char* loop;
    
    for(loop = buffer; loop < (buffer + length - 1); loop++) {
        xil_printf("%02X-", *loop);
    }
    xil_printf("%02X", *loop);
}

/* void printDottedDecimal(unsigned char* buffer, int length)
 *
 * Displays the contents of a byte array as a dot delimited string
 * to STDOUT
 */
void printDottedDecimal(unsigned char* buffer, int length)
{
    int loop;
    for(loop = 0; loop < length; loop++) {
        if(loop != 0) {
            xil_printf(".");
        }
        xil_printf("%d", buffer[loop]);
    }
}

/* void getMacAddr(unsigned char* address)
 *
 * Read and format a 6 byte MAC address from STDIN. The calling function
 * is responsible for allocating the memory required to store the MAC
 * address
 */
void getMacAddr(unsigned char* address)
{
    unsigned char character;
    unsigned char last[6];
    int index;
    
    // Ensure that the address buffer is zeroed
    memset(address, 0, 6);
     
    xil_printf("Please enter the final three bytes of the MAC address for your Xilinx\r\n");
    xil_printf("ML410 board. The MAC address is printed on a sticker located on the\r\n");
    xil_printf("top of the board, behind the Ethernet connectors\r\n\r\n");
    
    // Xilinx's MAC preamble is 00-0A-35, so all ML40x boards will have this MAC address.
    xil_printf("00-0A-35-");
    address[0] = 0x00;
    address[1] = 0x0A;
    address[2] = 0x35;
    
    // Loop through the MAC address, reading in characters as necessary
    index = 0;
    while(index < 6) {
        character = 0;
        character = XUartNs550_RecvByte(UART_BASEADDR);
        
        //Process numeric characters
        if((character > 47) && (character < 58)) {
            last[index] = character - 48;
            address[3 + (index / 2)] += character - 48;
            XUartNs550_SendByte(UART_BASEADDR, character);
            if(index % 2 == 0) {
                address[3 + (index / 2)] <<= 4;
            }
            if((index < 5) && (index % 2 == 1)) {
                XUartNs550_SendByte(UART_BASEADDR, '-');
            }
        }
        // Process uppercase A-F characters
        else if((character > 64) && (character < 71)) {
            last[index] = character - 55;
            address[3 + (index / 2)] += character - 55;
            XUartNs550_SendByte(UART_BASEADDR, character);
            if(index % 2 == 0) {
                address[3 + (index / 2)] <<= 4;
            }
            if((index < 5) && (index % 2 == 1)) {
                XUartNs550_SendByte(UART_BASEADDR, '-');
            }
        }
        // Process lowercase a-f characters
        else if((character > 96) && (character < 103)) {
            last[index] = character - 87;
            address[3 + (index / 2)] += character - 87;
            XUartNs550_SendByte(UART_BASEADDR, character - 32);
            if(index % 2 == 0) {
                address[3 + (index / 2)] <<= 4;
            }
            if((index < 5) && (index % 2 == 1)) {
                XUartNs550_SendByte(UART_BASEADDR, '-');
            }
        }
        // Process backspace characters
        else if((character == '\b') && (index > 0)) {
            index--;
            if(index % 2 == 1) {
                // It's necessary to do this as subsequent calls to
                // XUartNs550_SendByte() instead of xil_printf() because
                // xil_printf() does not encode the \b character
                XUartNs550_SendByte(UART_BASEADDR, '\b');
                XUartNs550_SendByte(UART_BASEADDR, ' ');
                XUartNs550_SendByte(UART_BASEADDR, '\b');
            }
            else {
                address[3 + (index / 2)] >>= 4;
            }            
            address[3 + (index / 2)] -= last[index];
            XUartNs550_SendByte(UART_BASEADDR, '\b');
            XUartNs550_SendByte(UART_BASEADDR, ' ');
            XUartNs550_SendByte(UART_BASEADDR, '\b');
            index--;
        }                
        // If this is an invalid character, ignore it
        else {
            index--;
        }
        
        index++;
    }
    
    xil_printf("\r\nIs the MAC address ");
    printByteDashedHex(address, 6);
    xil_printf(" correct? (y/n)");
    
    // Loop until we get a valid response
    while(1) {
        character = XUartNs550_RecvByte(UART_BASEADDR);
        if((character == 'Y') || (character == 'y')) {
            xil_printf("\r\n");
            break;
        }
        else if((character == 'N') || (character == 'n')) {
            xil_printf("\r\n");
            getMacAddr(address);
            break;
        }
    }
}

/* void clearScreen(void)
 *
 * Prints 24 newline characters to clear the display, assuming a standard 80x24
 * display
 */
void clearScreen(void)
{
    xil_printf("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\r\n");
}


/* int readIP(unsigned char* ip_address)
 *
 * Read and format an IP address from the console, and populate a four-byte buffer
 * containing the address. Returns true if the user has entered the address to
 * their satisfaction, and false otherwise
 */
int readIP(unsigned char* ip_address)
{
    unsigned char address[5];
    unsigned char character;
    int index;
    int charcount;
    
    xil_printf("\r\nPlease enter the IP address that you would like the ");
    xil_printf("ML410\r\nboard to use:\r\n");
    
    // Ensure that the buffers are initialized to zeros
    memset(ip_address, 0, 4);
    memset(address, 0, 5);    
    
    // Loop through the IP address to read in the values
    index = 0;
    charcount = 0;
    while(index < 4) {
       while(charcount < 3) {
           character = XUartNs550_RecvByte(UART_BASEADDR);
           // Process numeric (1-9) characters
           if((character > 47) && (character < 58)) {
               address[charcount] = character;
               XUartNs550_SendByte(UART_BASEADDR, character);
               charcount++;
           }
           // Process '.', '\r', and '\n' characters
           else if((character == '.') || (character == '\r') || (character == '\n')) {
               // Stop reading in additional characters for this segment
               if(charcount != 0) {
                   break;
               }
           }
       }
       // Delimit the fields with a '.' character
       if(index != 3) {
           XUartNs550_SendByte(UART_BASEADDR, '.');
       }
       
       // Format and store this byte of the address
       ip_address[index] = (unsigned char)atoi(address);
       memset(address, 0, 5);
       index++;
       charcount = 0;
    }
    
    // Check with the user whether or not the address is correct
    xil_printf("\r\nIs the address ");
    printDottedDecimal(ip_address, 4);
    xil_printf(" correct? (y/n)");
    
    // Loop until we get a valid response
    while(1) {
        character = XUartNs550_RecvByte(UART_BASEADDR);
        if((character == 'Y') || (character == 'y')) {
            xil_printf("\r\n");
            return TRUE;
        }
        else if((character == 'N') || (character == 'n')) {
            xil_printf("\r\n");
            return FALSE;
        }
    }     
}
    
/* void displayConfiguration(unsigned char* mac_address,
 *                           unsigned char* ip_address)
 *
 * Displays all of the currently set configuration options for the ping example
 * along with a header
 */
void displayConfiguration(unsigned char* mac_address,
                          unsigned char* ip_address)
{
    xil_printf("####################################################################\r\n");
	xil_printf("# Xilinx ML410 ICMP Echo (Ping) Demo Application v1.0              #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("# ICMP Echo (Ping) Server is now configured and running            #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("# Please note that if you have not purchased a license for the     #\r\n");
	xil_printf("# UART16550 core, you are currently using an evaluation license    #\r\n");
	xil_printf("# which will time out after approximately eight hours.             #\r\n");
	xil_printf("#                                                                  #\r\n");
	xil_printf("####################################################################\r\n");
	xil_printf("\r\nMAC Address: ");
	printByteDashedHex(mac_address, 6);
	xil_printf("\r\nIP Address : ");
	printDottedDecimal(ip_address, 4);
	xil_printf("\r\n\r\n");
}
